package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityAlreadyExistsException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.EntityNotFoundException;
import dev.fyloz.trial.colorrecipesexplorer.core.exception.model.ModelException;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MixType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.AbstractService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MixTypeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotNull;
import java.util.Optional;

@Service
public class MixTypeService extends AbstractService<MixType, MixTypeDao> {

    private MaterialService materialService;

    public MixTypeService() {
        super(MixType.class);
    }

    @Autowired
    public void setMixTypeDao(MixTypeDao mixTypeDao) {
        this.dao = mixTypeDao;
    }

    @Autowired
    public void setMaterialService(MaterialService materialService) {
        this.materialService = materialService;
    }

    /**
     * Vérifie si un type de mélange correspondant à un nom existe.
     *
     * @param name Le nom du type de mélange
     * @return Si un type de mélange correspondant au nom existe
     */
    public boolean existsByName(String name) {
        return dao.existsByName(name);
    }

    /**
     * Récupère le type de mélange correspondant à un nom.
     *
     * @param name Le nom du type de mélange
     * @return Le type de mélange correspondant au nom
     */
    public MixType getByName(String name) {
        Optional<MixType> found = dao.findByName(name);
        if (found.isEmpty()) throw new EntityNotFoundException(type, ModelException.IdentifierType.NAME, name);

        return found.get();
    }

    /**
     * Récupère le type de mélange correspondant à un produit.
     *
     * @param material Le produit du type de mélange
     * @return Le type de mélange correspondant au produit
     */
    public MixType getByMaterial(Material material) {
        Optional<MixType> found = dao.findByMaterial(material);
        if (found.isEmpty())
            throw new EntityNotFoundException(type, ModelException.IdentifierType.OTHER, MixType.IDENTIFIER_MATERIAL_NAME, material);

        return found.get();
    }

    /**
     * Crée un type de mélange.
     *
     * @param name         Le nom du type de mélange
     * @param materialType Le type de produit du type de mélange
     * @return Le type de mélange créé
     */
    public MixType create(String name, MaterialType materialType) {
        Material mixTypeMaterial = new Material(name, 0f, true, materialType);
        return new MixType(name, mixTypeMaterial);
    }

    @Override
    public MixType save(@NotNull MixType entity) {
        if (materialService.existsByName(entity.getName()))
            throw new EntityAlreadyExistsException(type, ModelException.IdentifierType.OTHER, MixType.IDENTIFIER_MATERIAL_NAME, entity.getName());

        return super.save(entity);
    }

    @Override
    public MixType update(MixType mixType) {

        return super.update(mixType);
    }
}
